/*
 * Tapioca XMPP Plugin
 * Copyright (c) 2005 INdT.
 * @author Luiz Augusto von Dentz <luiz.dentz@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif 

#include "xmpp_privacy.h"

#include <tapioca/base/tapioca_enums.h>
#include <tapioca/base/tapioca_errors.h>
#include <tapioca/base/tapioca_utils.h>

#include <string.h>

struct _XmppPrivacyPrivate {
    gboolean disposed;

    xmpp_privacy_t privacy;
};

static void     xmpp_privacy_dispose                (GObject *object);
static void     xmpp_privacy_request_lists          (TapiocaPrivacy *tp);
static void     xmpp_privacy_request_list_rules     (TapiocaPrivacy *tp,
                                                     const gchar *name);
static void     xmpp_privacy_get_default_list       (TapiocaPrivacy *tp,
                                                     const gchar **name);
static void     xmpp_privacy_set_default_list       (TapiocaPrivacy *tp,
                                                     const gchar *name);
static void     xmpp_privacy_get_active_list        (TapiocaPrivacy *tp,
                                                     const gchar **name);
static void     xmpp_privacy_set_active_list        (TapiocaPrivacy *tp,
                                                     const gchar *name);
static void     xmpp_privacy_add_list_rule          (TapiocaPrivacy *tp,
                                                     const gchar *list_name,
                                                     guint type,
                                                     const gchar *value,
                                                     guint action);
static void     xmpp_privacy_remove_list_rule       (TapiocaPrivacy *tp,
                                                     const gchar *list_name,
                                                     guint type,
                                                     const gchar *value,
                                                     guint action);
static void     lists_retrieved_cb                  (xmpp_privacy_t privacy,
                                                     void *data,
                                                     char **lists);
static void     rule_retrieved_cb                   (xmpp_privacy_t privacy,
                                                     void *data,
                                                     const char *list_name,
                                                     const char *type,
                                                     const char *value,
                                                     const char *action);

    
G_DEFINE_TYPE(XmppPrivacy, xmpp_privacy, TAPIOCA_TYPE_PRIVACY)

static void 
xmpp_privacy_class_init (XmppPrivacyClass *klass)
{
    GObjectClass *gobject_class;
    TapiocaPrivacyClass *tapioca_contact_list_class;

    gobject_class = (GObjectClass *) klass;
    tapioca_contact_list_class = (TapiocaPrivacyClass*) klass;
    xmpp_privacy_parent_class = g_type_class_peek_parent (klass);
    
    g_type_class_add_private (klass, sizeof (XmppPrivacyPrivate));

    tapioca_contact_list_class->request_lists = xmpp_privacy_request_lists;
    tapioca_contact_list_class->request_list_rules = xmpp_privacy_request_list_rules;
    tapioca_contact_list_class->get_default_list = xmpp_privacy_get_default_list;
    tapioca_contact_list_class->set_default_list = xmpp_privacy_set_default_list;
    tapioca_contact_list_class->get_active_list = xmpp_privacy_get_active_list;
    tapioca_contact_list_class->set_active_list = xmpp_privacy_set_active_list;
    tapioca_contact_list_class->add_list_rule = xmpp_privacy_add_list_rule;
    tapioca_contact_list_class->remove_list_rule = xmpp_privacy_remove_list_rule;
    gobject_class->dispose = xmpp_privacy_dispose;
}

static void 
xmpp_privacy_init (XmppPrivacy *xp)
{
    xp->priv = XMPP_PRIVACY_GET_PRIVATE (xp);
}

static void
xmpp_privacy_dispose (GObject *object)
{
    XmppPrivacy *xp = XMPP_PRIVACY (object);

    if (xp->priv->disposed) {
        /* If dispose did already run, return. */
        return;
    }
    /* Make sure dispose does not run twice. */
    xp->priv->disposed = TRUE;
   
    G_OBJECT_CLASS (xmpp_privacy_parent_class)->dispose (object);
}

XmppPrivacy *
xmpp_privacy_new (void)
{
    return XMPP_PRIVACY (g_object_new (XMPP_TYPE_PRIVACY, NULL));
}

void
xmpp_privacy_initialize (XmppPrivacy *xp, xmpp_privacy_t *privacy)
{
    g_debug ("initializing privacy");

    g_return_if_fail (privacy != NULL);

    xp->priv->privacy = privacy;
    xmpp_privacy_set_lists_retrieved_cb (privacy, lists_retrieved_cb, xp);
    xmpp_privacy_set_rule_retrieved_cb (privacy, rule_retrieved_cb, xp);
}

static void
xmpp_privacy_request_lists (TapiocaPrivacy *tp)
{
    XmppPrivacy *xp = XMPP_PRIVACY (tp);

    g_return_if_fail (xp->priv->privacy);
    
    xmpp_privacy_list_request_lists (xp->priv->privacy);
}
static void
xmpp_privacy_request_list_rules (TapiocaPrivacy *tp,
                                 const gchar *name)
{
    XmppPrivacy *xp = XMPP_PRIVACY (tp);

    g_return_if_fail (xp->priv->privacy);

    xmpp_privacy_list_request_rules (xp->priv->privacy, name);
}
static void
xmpp_privacy_get_default_list (TapiocaPrivacy *tp,
                               const gchar **name)
{
    XmppPrivacy *xp = XMPP_PRIVACY (tp);

    g_return_if_fail (xp->priv->privacy);

    xmpp_privacy_list_get_default (xp->priv->privacy);
}
static void
xmpp_privacy_set_default_list (TapiocaPrivacy *tp,
                               const gchar *name)
{
    XmppPrivacy *xp = XMPP_PRIVACY (tp);

    g_return_if_fail (xp->priv->privacy);

    xmpp_privacy_list_set_default (xp->priv->privacy, name);
}
static void
xmpp_privacy_get_active_list (TapiocaPrivacy *tp,
                              const gchar **name)
{
    XmppPrivacy *xp = XMPP_PRIVACY (tp);

    g_return_if_fail (xp->priv->privacy);

    xmpp_privacy_list_get_active (xp->priv->privacy);
}
static void
xmpp_privacy_set_active_list (TapiocaPrivacy *tp,
                              const gchar *name)
{
    XmppPrivacy *xp = XMPP_PRIVACY (tp);

    g_return_if_fail (xp->priv->privacy);

    xmpp_privacy_list_set_active (xp->priv->privacy, name);
}
static void
xmpp_privacy_add_list_rule (TapiocaPrivacy *tp,
                            const gchar *list_name,
                            guint type,
                            const gchar *value,
                            guint action)
{
    XmppPrivacy *xp = XMPP_PRIVACY (tp);
    const gchar *type_str = NULL;
    const gchar *action_str = NULL;

    g_return_if_fail (xp->priv->privacy);

    switch (type) {
        case TAPIOCA_PRIVACY_RULE_ADDRESS:
            type_str = g_strdup ("jid");
            break;
        case TAPIOCA_PRIVACY_RULE_GROUP:
            type_str = g_strdup ("group");
            break;
        case TAPIOCA_PRIVACY_RULE_SUBSCRIPTION:
            type_str = g_strdup ("subscription");
            break;
    }

    switch (action) {
        case TAPIOCA_PRIVACY_RULE_ACTION_ACCEPT:
            action_str = g_strdup ("allow");
            break;
        case TAPIOCA_PRIVACY_RULE_ACTION_BLOCK:
            action_str = g_strdup ("deny");
            break;
    }
    
    xmpp_privacy_list_add_rule (xp->priv->privacy, list_name, type_str, value, action_str);
}
static void
xmpp_privacy_remove_list_rule (TapiocaPrivacy *tp,
                               const gchar *list_name,
                               guint type,
                               const gchar *value,
                               guint action)
{
    XmppPrivacy *xp = XMPP_PRIVACY (tp);
    const gchar *type_str = NULL;
    const gchar *action_str = NULL;

    g_return_if_fail (xp->priv->privacy);

    switch (type) {
        case TAPIOCA_PRIVACY_RULE_ADDRESS:
            type_str = g_strdup ("jid");
            break;
        case TAPIOCA_PRIVACY_RULE_GROUP:
            type_str = g_strdup ("group");
            break;
        case TAPIOCA_PRIVACY_RULE_SUBSCRIPTION:
            type_str = g_strdup ("subscription");
            break;
    }

    switch (action) {
        case TAPIOCA_PRIVACY_RULE_ACTION_ACCEPT:
            action_str = g_strdup ("allow");
            break;
        case TAPIOCA_PRIVACY_RULE_ACTION_BLOCK:
            action_str = g_strdup ("deny");
            break;
    }

    xmpp_privacy_list_remove_rule (xp->priv->privacy, list_name, type_str, value, action_str);
}
static void
lists_retrieved_cb (xmpp_privacy_t privacy,
                    void *data,
                    char **lists)
{
    TapiocaPrivacy *tp = TAPIOCA_PRIVACY (data);

    tapioca_privacy_signal_lists_retrieved (tp, lists);
}
static void
rule_retrieved_cb (xmpp_privacy_t privacy,
                   void *data,
                   const char *list_name,
                   const char *type,
                   const char *value,
                   const char *action)
{
    TapiocaPrivacy *tp = TAPIOCA_PRIVACY (data);
    guint rule_type = -1;
    guint rule_action = -1;

    if (!strcmp (type,"jid"))
        rule_type = TAPIOCA_PRIVACY_RULE_ADDRESS;
    else if (!strcmp (type,"group"))
        rule_type = TAPIOCA_PRIVACY_RULE_GROUP;
    else if (!strcmp (type,"subscription"))
        rule_type = TAPIOCA_PRIVACY_RULE_SUBSCRIPTION;

    if (!strcmp (action,"allow"))
        rule_action = TAPIOCA_PRIVACY_RULE_ACTION_ACCEPT;
    else if(!strcmp (action,"deny"))
        rule_action = TAPIOCA_PRIVACY_RULE_ACTION_BLOCK;

    tapioca_privacy_signal_rule_retrieved (tp, list_name, rule_type, value, rule_action);
}
