/*
 * Copyright (c) 2005 INdT.
 * @author Luiz Augusto von Dentz <luiz.dentz@indt.org.br> 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "xmpp_privacy_list.h"

#include <talk/xmpp/xmppclient.h>

XmppPrivacyList::XmppPrivacyList(buzz::Task *parent) 
    : buzz::XmppTask(parent, buzz::XmppEngine::HL_TYPE)
{
    active_list_rules_ = NULL;
}

XmppPrivacyList::~XmppPrivacyList()
{
}

void XmppPrivacyList::RequestLists()
{
    SendStanza(GenerateXml());
}
void XmppPrivacyList::RequestRules(const std::string &name)
{
    buzz::XmlElement *list = new buzz::XmlElement(buzz::QN_PRIVACY_LIST);
    
    list->AddAttr(buzz::QN_NAME, name);
        
    SendStanza(GenerateXml(list, buzz::STR_GET));
}
void XmppPrivacyList::SetDefault(const std::string &name)
{
    buzz::XmlElement *list = new buzz::XmlElement(buzz::QN_PRIVACY_DEFAULT);
    
    list->AddAttr(buzz::QN_NAME, name);
            
    SendStanza(GenerateXml(list));
}
void XmppPrivacyList::SetActive(const std::string &name)
{
    buzz::XmlElement *list = new buzz::XmlElement(buzz::QN_PRIVACY_ACTIVE);
    
    list->AddAttr(buzz::QN_NAME, name);
            
    SendStanza(GenerateXml(list));
}
void XmppPrivacyList::AddRule(const std::string &name, const std::string &type,
    const std::string &value, const std::string &action)
{
    buzz::XmlElement *item = new buzz::XmlElement(buzz::QN_PRIVACY_ITEM);
    
    item->AddAttr(buzz::QN_TYPE, type);
    if (value != buzz::STR_EMPTY) 
        item->AddAttr(buzz::QN_VALUE, value);
    item->AddAttr(buzz::QN_ACTION, action);
    
    if (!active_list_rules_ || !active_list_rules_->HasAttr(buzz::QN_NAME)) {
        RequestLists();
        buzz::XmlElement *list = new buzz::XmlElement(buzz::QN_PRIVACY_LIST);   

        item->AddAttr(buzz::QN_ORDER, std::string("1"));

        list->AddAttr(buzz::QN_NAME, name);
        list->AddElement(item);
        active_list_rules_ = list;
    }
    else if (active_list_rules_->Attr(buzz::QN_NAME) == name)
        active_list_rules_->AddElement(item);
    
    SendStanza(GenerateXml(active_list_rules_));
}
void XmppPrivacyList::RemoveRule(const std::string &name, const std::string &type,
    const std::string &value, const std::string &action)
{    
    buzz::XmlElement *item;

    if (!active_list_rules_)
        return;
    
    if (active_list_rules_->Attr(buzz::QN_NAME) == name) {
        for (item = active_list_rules_->FirstNamed(buzz::QN_PRIVACY_ITEM); item;
            item = item->NextNamed(buzz::QN_PRIVACY_ITEM)) {
            if (item->Attr(buzz::QN_TYPE) == type && item->Attr(buzz::QN_VALUE) == value 
                && item->Attr(buzz::QN_ACTION) == action) {
                item->ClearChildren();
                break;
            }
        }
    }
        
    SendStanza(GenerateXml(active_list_rules_));    
}

bool XmppPrivacyList::HandleStanza(const buzz::XmlElement *stanza) 
{
    /* Only handle IQ stanzas with result type and <query xmlns='jabber:iq:roster'/>*/
    if (stanza->Name() == buzz::QN_IQ) {
        if ((stanza->Attr(buzz::QN_TYPE) == buzz::STR_ERROR) ||
            stanza->FirstElement() == NULL ||
            stanza->FirstElement()->Name() != buzz::QN_PRIVACY_QUERY || 
    	    stanza->FirstElement()->Attr(buzz::QN_XMLNS) !=  buzz::Constants::ns_privacy())
            return false;
    }
    else
        return false;
    QueueStanza(stanza);
    return true;
}

int XmppPrivacyList::ProcessStart() 
{
    const buzz::XmlElement *stanza = NextStanza();
    const buzz::XmlElement *active_list;
    const buzz::XmlElement *default_list;
    const buzz::XmlElement *query;
    const buzz::XmlElement *list;
    const buzz::XmlElement *item;
    int cont;
    char *lists[256];
    
    if (stanza == NULL)
        return STATE_BLOCKED;

    crit_.Enter();
    query = stanza->FirstNamed(buzz::QN_PRIVACY_QUERY);
    if ((active_list = query->FirstNamed(buzz::QN_PRIVACY_ACTIVE)) && 
        (default_list = query->FirstNamed(buzz::QN_PRIVACY_DEFAULT))) {
        active_list_ = active_list->Attr(buzz::QN_NAME);
        default_list_ = active_list->Attr(buzz::QN_NAME);
    }
            
    for (cont = 0, list = query->FirstNamed(buzz::QN_PRIVACY_LIST); list; cont++,
        list = list->NextNamed(buzz::QN_PRIVACY_LIST)) {
        lists[cont] = strdup (list->Attr(buzz::QN_NAME).c_str());
        for (item = list->FirstNamed(buzz::QN_PRIVACY_ITEM); item;
            item = item->NextNamed(buzz::QN_PRIVACY_ITEM)) {
            rule_cb_ ((xmpp_privacy_t) this, rule_data_,
                list->Attr(buzz::QN_NAME).c_str(),
                item->Attr(buzz::QN_TYPE).c_str(),
                item->Attr(buzz::QN_VALUE).c_str(),
                item->Attr(buzz::QN_ACTION).c_str());
        }
    }

    if (cont > 1)
        lists_cb_ ((xmpp_privacy_t) this, lists_data_, lists);
        
    for (; cont; cont--)
        free (lists[cont-1]);
        
    return STATE_START;
}

buzz::XmlElement *XmppPrivacyList::GenerateXml(buzz::XmlElement *list, const std::string &type)
{
    buzz::XmlElement *result = new buzz::XmlElement(buzz::QN_IQ);
    buzz::XmlElement *query = new buzz::XmlElement(buzz::QN_PRIVACY_QUERY);
    
    query->AddAttr(buzz::QN_XMLNS, buzz::Constants::ns_privacy());

    if (list) {
        query->AddElement(list);        
        result->AddAttr(buzz::QN_TYPE, type);
    }
    else
        result->AddAttr(buzz::QN_TYPE, buzz::STR_GET);
        
    result->AddAttr(buzz::QN_FROM, GetClient()->jid().Str());
    result->AddAttr(buzz::QN_ID, GenerateId());
    result->AddElement(query);
        
    return result;
}
